@@ -1,23 +1,11 @@
-baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol, grid=TRUE, progbar=FALSE, ...) {
+baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol="ids", grid=TRUE, progbar=FALSE, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(x, "rma"))
-      stop(mstyle$stop("Argument 'x' must be an object of class \"rma\"."))
-
-   if (inherits(x, "rma.glmm"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.glmm\"."))
-
-   if (inherits(x, "rma.mv"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.mv\"."))
-
-   if (inherits(x, "robust.rma"))
-      stop(mstyle$stop("Method not available for objects of class \"robust.rma\"."))
-
-   if (inherits(x, "rma.ls"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.ls\"."))
+   .chkclass(class(x), must="rma", notav=c("rma.glmm", "rma.mv", "robust.rma", "rma.ls", "rma.uni.selmodel"))
 
    na.act <- getOption("na.action")
+   on.exit(options(na.action=na.act), add=TRUE)
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
@@ -25,6 +13,15 @@ baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol, grid=TRUE, progba
    if (x$k == 1)
       stop(mstyle$stop("Stopped because k = 1."))
 
+   ### grid argument can either be a logical or a color
+
+   if (is.logical(grid))
+      gridcol <- "lightgray"
+   if (is.character(grid)) {
+      gridcol <- grid
+      grid <- TRUE
+   }
+
    #########################################################################
 
    ### set up vectors to store results in
@@ -36,33 +33,36 @@ baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol, grid=TRUE, progba
 
    pred.full <- x$X.f %*% x$beta
 
+   ### elements that need to be returned
+
+   outlist <- "coef.na=coef.na, beta=beta, vb=vb"
+
    ### note: skipping NA cases
    ### also: it is possible that model fitting fails, so that generates more NAs (these NAs will always be shown in output)
 
    if (progbar)
-      pbar <- txtProgressBar(min=0, max=x$k.f, style=3)
+      pbar <- pbapply::startpb(min=0, max=x$k.f)
 
    for (i in seq_len(x$k.f)) {
 
       if (progbar)
-         setTxtProgressBar(pbar, i)
+         pbapply::setpb(pbar, i)
 
       if (!x$not.na[i])
          next
 
       if (inherits(x, "rma.uni"))
-         res <- try(suppressWarnings(rma.uni(x$yi.f, x$vi.f, weights=x$weights.f, mods=x$X.f, intercept=FALSE, method=x$method, weighted=x$weighted, test=x$test, tau2=ifelse(x$tau2.fix, x$tau2, NA), control=x$control, subset=-i)), silent=TRUE)
-
+         res <- try(suppressWarnings(.do.call(rma.uni, yi=x$yi.f, vi=x$vi.f, weights=x$weights.f, mods=x$X.f, intercept=FALSE, method=x$method, weighted=x$weighted, test=x$test, level=x$level, tau2=ifelse(x$tau2.fix, x$tau2, NA), control=x$control, subset=-i, skipr2=TRUE, outlist=outlist)), silent=TRUE)
       if (inherits(x, "rma.mh")) {
          if (is.element(x$measure, c("RR","OR","RD"))) {
-            res <- try(suppressWarnings(rma.mh(ai=x$ai.f, bi=x$bi.f, ci=x$ci.f, di=x$di.f, measure=x$measure, add=x$add, to=x$to, drop00=x$drop00, correct=x$correct, subset=-i)), silent=TRUE)
+            res <- try(suppressWarnings(.do.call(rma.mh, ai=x$ai.f, bi=x$bi.f, ci=x$ci.f, di=x$di.f, measure=x$measure, add=x$add, to=x$to, drop00=x$drop00, correct=x$correct, level=x$level, subset=-i, outlist=outlist)), silent=TRUE)
          } else {
-            res <- try(suppressWarnings(rma.mh(x1i=x$x1i.f, x2i=x$x2i.f, t1i=x$t1i.f, t2i=x$t2i.f, measure=x$measure, add=x$add, to=x$to, drop00=x$drop00, correct=x$correct, subset=-i)), silent=TRUE)
+            res <- try(suppressWarnings(.do.call(rma.mh, x1i=x$x1i.f, x2i=x$x2i.f, t1i=x$t1i.f, t2i=x$t2i.f, measure=x$measure, add=x$add, to=x$to, drop00=x$drop00, correct=x$correct, level=x$level, subset=-i, outlist=outlist)), silent=TRUE)
          }
       }
 
       if (inherits(x, "rma.peto"))
-         res <- try(suppressWarnings(rma.peto(ai=x$ai.f, bi=x$bi.f, ci=x$ci.f, di=x$di.f, add=x$add, to=x$to, drop00=x$drop00, subset=-i)), silent=TRUE)
+         res <- try(suppressWarnings(.do.call(rma.peto, ai=x$ai.f, bi=x$bi.f, ci=x$ci.f, di=x$di.f, add=x$add, to=x$to, drop00=x$drop00, level=x$level, subset=-i)), silent=TRUE)
 
       if (inherits(res, "try-error"))
          next
@@ -79,7 +79,7 @@ baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol, grid=TRUE, progba
    }
 
    if (progbar)
-      close(pbar)
+      pbapply::closepb(pbar)
 
    yhati <- (delpred - pred.full)^2 / vdelpred
 
@@ -88,7 +88,7 @@ baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol, grid=TRUE, progba
    ### x-axis values (use 'na.pass' to make sure we get a vector of length k.f)
 
    options(na.action = "na.pass")
-   xhati <- 1/(x$tau2.f + x$vi.f) * resid(x)^2
+   xhati <- resid(x)^2 / (x$tau2.f + x$vi.f)
    options(na.action = na.act)
 
    #########################################################################
@@ -99,7 +99,7 @@ baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol, grid=TRUE, progba
       cex <- 0.8
 
    if (missing(xlab)) {
-      if (x$method == "FE") {
+      if (is.element(x$method, c("FE","EE","CE"))) {
          xlab <- ifelse(x$int.only, "Contribution to Overall Heterogeneity", "Contribution to Residual Heterogeneity")
       } else {
          xlab <- "Squared Pearson Residual"
@@ -123,27 +123,36 @@ baujat.rma <- function(x, xlim, ylim, xlab, ylab, cex, symbol, grid=TRUE, progba
 
    ### add grid (and redraw box)
 
-   if (grid) {
-      grid()
+   if (.isTRUE(grid)) {
+      grid(col=gridcol)
       box(...)
    }
 
-   ### add points/symbols
+   if (is.numeric(symbol)) {
 
-   if (missing(symbol))
-      symbol <- "ids"
+      if (length(symbol) == 1L)
+         symbol <- rep(symbol, x$k.all)
 
-   if (is.numeric(symbol))
-      points(xhati, yhati, cex=cex, pch=symbol, ...)
+      if (length(symbol) != x$k.all)
+         stop(mstyle$stop(paste0("Length of the 'symbol' argument (", length(symbol), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-   if (is.character(symbol) && symbol == "ids")
+      if (!is.null(x$subset))
+         symbol <- symbol[x$subset]
+
+      points(x=xhati, y=yhati, cex=cex, pch=symbol, ...)
+
+   }
+
+   if (is.character(symbol) && symbol=="ids")
       text(xhati, yhati, x$ids, cex=cex, ...)
 
-   if (is.character(symbol) && symbol == "slab")
+   if (is.character(symbol) && symbol=="slab")
       text(xhati, yhati, x$slab, cex=cex, ...)
 
    #########################################################################
 
-   invisible(data.frame(x=xhati[x$not.na], y=yhati[x$not.na], ids=x$ids[x$not.na], slab=x$slab[x$not.na]))
+   sav <- data.frame(x=xhati[x$not.na], y=yhati[x$not.na], ids=x$ids[x$not.na], slab=x$slab[x$not.na], stringsAsFactors=FALSE)
+
+   invisible(sav)
 
 }
