@@ -24,7 +24,7 @@ blup.rma.uni <- function(x, level, digits, transf, targs, ...) {
    if (missing(targs))
       targs <- NULL
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
 
    if (is.element(x$test, c("knha","adhoc","t"))) {
       crit <- qt(level/2, df=x$ddf, lower.tail=FALSE)
