@@ -2,8 +2,10 @@ confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(object, "rma.mh"))
-      stop(mstyle$stop("Argument 'object' must be an object of class \"rma.mh\"."))
+   .chkclass(class(object), must="rma.mh")
+
+   if (!missing(parm))
+      warning(mstyle$warning("Argument 'parm' (currently) ignored."), call.=FALSE)
 
    x <- object
 
@@ -22,9 +24,16 @@ confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
    if (missing(targs))
       targs <- NULL
 
+   ddd <- list(...)
+
+   .chkdots(ddd, c("time"))
+
+   if (.isTRUE(ddd$time))
+      time.start <- proc.time()
+
    #########################################################################
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
    crit  <- qnorm(level/2, lower.tail=FALSE)
 
    beta <- x$beta
@@ -48,6 +57,12 @@ confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
       }
    }
 
+   ### make sure order of intervals is always increasing
+
+   tmp <- .psort(ci.lb, ci.ub)
+   ci.lb <- tmp[,1]
+   ci.ub <- tmp[,2]
+
    #########################################################################
 
    res <- cbind(estimate=beta, ci.lb, ci.ub)
@@ -56,6 +71,11 @@ confint.rma.mh <- function(object, parm, level, digits, transf, targs, ...) {
 
    res$digits <- digits
 
+   if (.isTRUE(ddd$time)) {
+      time.end <- proc.time()
+      .print.time(unname(time.end - time.start)[3])
+   }
+
    class(res) <- "confint.rma"
    return(res)
 
