@@ -1,26 +1,20 @@
-### Note: There is code below to obtain a profile likelihood CI for tau^2, but then
-### I may have to introduce a 'type' argument to specify which type of CI to obtain.
-### Actually, what would be most consistent is this:
-### if method='ML/REML':    profile likelihood (PL) CI (based on the ML/REML likelihood)
-### if method='EB/PM/PMM':  Q-profile (QP) CI
-### if method='GENQ/GENQM': generalized Q-statistic (GENQ) CI (which also covers method='DL/HE' as special cases)
-### if method='SJ':         method by Sidik & Jonkman (2005) (but this performs poorly, except if tau^2 is very large)
-### if method='HS':         not sure since this is an ad-hoc estimator with no obvious underlying statistical principle
-### Also could in principle compute Wald-type CIs (but those perform poorly except when k is very large).
-### But it may be a bit late to change how the function works (right now, type="GENQ" if method="GENQ/GENQM" and type="QP" otherwise).
+# What would be most consistent is this:
+# if method='ML/REML':    profile likelihood (PL) CI (based on the ML/REML likelihood)
+# if method='EB/PM/PMM':  Q-profile (QP) CI
+# if method='GENQ/GENQM': generalized Q-statistic (GENQ) CI (which also covers method='DL/HE' as special cases)
+# if method='SJ':         method by Sidik & Jonkman (2005) (but this performs poorly, except if tau^2 is very large)
+# if method='HS':         not sure since this is an ad-hoc estimator with no obvious underlying statistical principle
+# Also could in principle compute Wald-type CIs (but those perform poorly except when k is very large).
+# Too late to change how the function works (right now, type="GENQ" if method="GENQ/GENQM" and type="QP" otherwise).
 
-confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digits, transf, targs, verbose=FALSE, control, ...) {
+confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, type, digits, transf, targs, verbose=FALSE, control, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(object, "rma.uni"))
-      stop(mstyle$stop("Argument 'object' must be an object of class \"rma.uni\"."))
+   .chkclass(class(object), must="rma.uni", notav=c("robust.rma", "rma.ls"))
 
-   if (inherits(object, "robust.rma"))
-      stop(mstyle$stop("Method not available for objects of class \"robust.rma\"."))
-
-   if (inherits(object, "rma.ls"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.ls\"."))
+   if (!missing(parm))
+      warning(mstyle$warning("Argument 'parm' (currently) ignored."), call.=FALSE)
 
    x <- object
 
@@ -53,23 +47,33 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
    if (!fixed && !random)
       stop(mstyle$stop("At least one of the arguments 'fixed' and 'random' must be TRUE."))
 
-   if (x$method == "GENQ" || x$method == "GENQM") {
-      type <- "GENQ"
-   } else {
-      type <- "QP"
+   level <- .level(level)
+
+   ddd <- list(...)
+
+   .chkdots(ddd, c("time", "xlim", "extint"))
+
+   if (.isTRUE(ddd$time))
+      time.start <- proc.time()
+
+   if (!is.null(ddd$xlim)) {
+      if (length(ddd$xlim) != 2L)
+         stop(mstyle$stop("Argument 'xlim' should be a vector of length 2."))
+      control$tau2.min <- ddd$xlim[1]
+      control$tau2.max <- ddd$xlim[2]
    }
 
-   #if (missing(type)) {
-   #   if (x$method == "GENQ" || x$method == "GENQM") {
-   #      type <- "GENQ"
-   #   } else {
-   #      type <- "QP"
-   #   }
-   #} else {
-   #   type <- match.arg(type, c("QP", "GENQ", "PL"))
-   #}
-
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   if (missing(type)) {
+      if (x$method == "GENQ" || x$method == "GENQM") {
+         type <- "GENQ"
+      } else {
+         type <- "QP"
+      }
+   } else {
+      #type <- match.arg(type, c("QP", "GENQ", "PL"))
+      if (!is.element(type, c("QP", "GENQ", "PL")))
+         stop(mstyle$stop("Unknown 'type' specified."))
+   }
 
    #########################################################################
    #########################################################################
@@ -80,21 +84,21 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
       if (k == 1)
          stop(mstyle$stop("Stopped because k = 1."))
 
-      if (x$method == "FE")
+      if (is.element(x$method, c("FE","EE","CE")))
          stop(mstyle$stop("Model does not contain a random-effects component."))
 
       if (x$tau2.fix)
          stop(mstyle$stop("Model does not contain an estimated random-effects component."))
 
       if (type == "GENQ" && !(is.element(x$method, c("GENQ","GENQM"))))
-         stop(mstyle$stop("Model must be fitted with 'method=\"GENQ\" or 'method=\"GENQM\" to use this option."))
+         stop(mstyle$stop("Model must be fitted with method=\"GENQ\" or method=\"GENQM\" to use this option."))
 
       ######################################################################
 
       ### set control parameters for uniroot() and possibly replace with user-defined values
       ### set tau2.min and tau2.max and possibly replace with user-defined values
       ### note: default tau2.min is smaller of 0 or tau2, since tau2 could in principle be negative
-      ### note: default tau2.max must be larger than tau2 and tau2.min and really should be much larger (at least 100 ...)
+      ### note: default tau2.max must be larger than tau2 and tau2.min and really should be much larger (at least 100)
 
       tau2.min <- ifelse(is.null(x$control$tau2.min), min(0, x$tau2), x$control$tau2.min)
       tau2.max <- ifelse(is.null(x$control$tau2.max), max(100, x$tau2*10, tau2.min*10), x$control$tau2.max)
@@ -103,7 +107,8 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
 
       con <- list(tol=.Machine$double.eps^0.25, maxiter=1000, tau2.min=tau2.min, tau2.max=tau2.max, verbose=FALSE)
 
-      con[pmatch(names(control), names(con))] <- control
+      con.pos <- pmatch(names(control), names(con))
+      con[c(na.omit(con.pos))] <- control[!is.na(con.pos)]
 
       if (verbose)
          con$verbose <- verbose
@@ -131,7 +136,7 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
       if (type == "QP") {
 
          if (!x$allvipos)
-            stop(mstyle$stop("Cannot compute confidence interval for the amount of (residual)\n  heterogeneity with non-positive sampling variances in the data."))
+            stop(mstyle$stop("Cannot compute CI for tau^2 when there are non-positive sampling variances in the data."))
 
          crit.u <- qchisq(level/2, k-p, lower.tail=FALSE) ### upper critical chi^2 value for df = k-p
          crit.l <- qchisq(level/2, k-p, lower.tail=TRUE)  ### lower critical chi^2 value for df = k-p
@@ -385,9 +390,9 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
          ### get diff value when setting component to tau2.min; this value should be positive (i.e., discrepancy must be larger than critical value)
          ### if it is not, then the lower bound must be below tau2.min
 
-         res <- try(.profile.rma.uni(val = con$tau2.min, obj=x, CI=TRUE, objective=objective, verbose=verbose), silent=TRUE)
+         res <- try(.profile.rma.uni(con$tau2.min, obj=x, confint=TRUE, objective=objective, verbose=verbose), silent=TRUE)
 
-         if (!inherits(res, "try-error")) {
+         if (!inherits(res, "try-error") && !is.na(res)) {
 
             if (res < 0) {
 
@@ -399,7 +404,11 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
 
             } else {
 
-               res <- try(uniroot(.profile.rma.uni, interval=c(con$tau2.min, x$tau2), tol=con$tol, maxiter=con$maxiter, obj=x, CI=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
+               if (.isTRUE(ddd$extint)) {
+                  res <- try(uniroot(.profile.rma.uni, interval=c(con$tau2.min, x$tau2), tol=con$tol, maxiter=con$maxiter, extendInt="downX", obj=x, confint=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
+               } else {
+                  res <- try(uniroot(.profile.rma.uni, interval=c(con$tau2.min, x$tau2), tol=con$tol, maxiter=con$maxiter, obj=x, confint=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
+               }
 
                ### check if uniroot method converged
 
@@ -421,11 +430,11 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
          ### get diff value when setting component to tau2.max; this value should be positive (i.e., discrepancy must be larger than critical value)
          ### if it is not, then the upper bound must be above tau2.max
 
-         res <- try(.profile.rma.uni(val = con$tau2.max, obj=x, CI=TRUE, objective=objective, verbose=verbose), silent=TRUE)
+         res <- try(.profile.rma.uni(con$tau2.max, obj=x, confint=TRUE, objective=objective, verbose=verbose), silent=TRUE)
 
-         if (!inherits(res, "try-error")) {
+         if (!inherits(res, "try-error") && !is.na(res)) {
 
-            if (res < 0) {
+            if (!.isTRUE(ddd$extint) && res < 0) {
 
                tau2.ub <- con$tau2.max
                ub.conv <- TRUE
@@ -433,7 +442,11 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
 
             } else {
 
-               res <- try(uniroot(.profile.rma.uni, interval=c(x$tau2, con$tau2.max), tol=con$tol, maxiter=con$maxiter, obj=x, CI=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
+               if (.isTRUE(ddd$extint)) {
+                  res <- try(uniroot(.profile.rma.uni, interval=c(x$tau2, con$tau2.max), tol=con$tol, maxiter=con$maxiter, extendInt="upX", obj=x, confint=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
+               } else {
+                  res <- try(uniroot(.profile.rma.uni, interval=c(x$tau2, con$tau2.max), tol=con$tol, maxiter=con$maxiter, obj=x, confint=TRUE, objective=objective, verbose=verbose, check.conv=TRUE)$root, silent=TRUE)
+               }
 
                ### check if uniroot method converged
 
@@ -455,29 +468,25 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
       ######################################################################
 
       if (!lb.conv)
-         warning(mstyle$warning("Error in iterative search for the lower bound."))
+         warning(mstyle$warning("Error in iterative search for the lower bound."), call.=FALSE)
 
       if (!ub.conv)
-         warning(mstyle$warning("Error in iterative search for the upper bound."))
+         warning(mstyle$warning("Error in iterative search for the upper bound."), call.=FALSE)
 
       #if (lb.sign == "<" && con$tau2.min > 0)
-      #   warning(mstyle$warning("Lower bound < tau2.min. Try decreasing tau2.min (via the 'control' argument)."))
+      #   warning(mstyle$warning("Lower bound < tau2.min. Try decreasing tau2.min (via the 'control' argument)."), call.=FALSE)
       #if (ub.sign == ">")
-      #   warning(mstyle$warning("Upper bound > tau2.max. Try increasing tau2.max (via the 'control' argument)."))
+      #   warning(mstyle$warning("Upper bound > tau2.max. Try increasing tau2.max (via the 'control' argument)."), call.=FALSE)
 
       ######################################################################
 
-      W      <- diag(1/vi, nrow=k, ncol=k)
-      stXWX  <- .invcalc(X=X, W=W, k=k)
-      P      <- W - W %*% X %*% stXWX %*% crossprod(X,W)
-      vi.avg <- (k-p) / .tr(P)
-      I2.lb  <- 100 * tau2.lb / (vi.avg + tau2.lb)
-      I2.ub  <- 100 * tau2.ub / (vi.avg + tau2.ub)
-      H2.lb  <- tau2.lb / vi.avg + 1
-      H2.ub  <- tau2.ub / vi.avg + 1
+      I2.lb <- 100 * tau2.lb / (x$vt + tau2.lb)
+      I2.ub <- 100 * tau2.ub / (x$vt + tau2.ub)
+      H2.lb <- tau2.lb / x$vt + 1
+      H2.ub <- tau2.ub / x$vt + 1
 
       tau2 <- c(x$tau2, tau2.lb, tau2.ub)
-      tau  <- sqrt(c(ifelse(x$tau2 >= 0, x$tau2, NA), ifelse(tau2.lb >= 0, tau2.lb, NA), ifelse(tau2.ub >= 0, tau2.ub, NA)))
+      tau  <- sqrt(c(ifelse(x$tau2 >= 0, x$tau2, NA_real_), ifelse(tau2.lb >= 0, tau2.lb, NA_real_), ifelse(tau2.ub >= 0, tau2.ub, NA_real_)))
       I2   <- c(x$I2, I2.lb, I2.ub)
       H2   <- c(x$H2, H2.lb, H2.ub)
 
@@ -493,7 +502,7 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
    if (fixed) {
 
       if (is.element(x$test, c("knha","adhoc","t"))) {
-         crit <- qt(level/2, df=x$dfs, lower.tail=FALSE)
+         crit <- qt(level/2, df=x$ddf, lower.tail=FALSE)
       } else {
          crit <- qnorm(level/2, lower.tail=FALSE)
       }
@@ -514,6 +523,12 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
          }
       }
 
+      ### make sure order of intervals is always increasing
+
+      tmp <- .psort(ci.lb, ci.ub)
+      ci.lb <- tmp[,1]
+      ci.ub <- tmp[,2]
+
       res.fixed <- cbind(estimate=beta, ci.lb=ci.lb, ci.ub=ci.ub)
       rownames(res.fixed) <- rownames(x$beta)
 
@@ -540,6 +555,11 @@ confint.rma.uni <- function(object, parm, level, fixed=FALSE, random=TRUE, digit
       res$tau2.min <- con$tau2.min
    }
 
+   if (.isTRUE(ddd$time)) {
+      time.end <- proc.time()
+      .print.time(unname(time.end - time.start)[3])
+   }
+
    class(res) <- "confint.rma"
    return(res)
 
