@@ -1,10 +1,10 @@
-forest.rma <- function(x,
+forest.rma       <- function(x,
 annotate=TRUE, addfit=TRUE, addpred=FALSE, showweights=FALSE, header=FALSE,
-xlim, alim, olim, ylim, top=3, at, steps=5, level=x$level, refline=0, digits=2L, width,
+xlim, alim, olim, ylim, at, steps=5, level=x$level, refline=0, digits=2L, width,
 xlab, slab, mlab, ilab, ilab.xpos, ilab.pos, order,
 transf, atransf, targs, rows,
-efac=1, pch=15, psize, plim=c(0.5,1.5), colout, col, border,
-lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
+efac=1, pch, psize, plim=c(0.5,1.5), colout, col, border,
+lty, fonts, cex, cex.lab, cex.axis, ...) {
 
    #########################################################################
 
@@ -13,7 +13,7 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    .chkclass(class(x), must="rma", notav="rma.ls")
 
    na.act <- getOption("na.action")
-   on.exit(options(na.action=na.act))
+   on.exit(options(na.action=na.act), add=TRUE)
 
    if (!is.element(na.act, c("na.omit", "na.exclude", "na.fail", "na.pass")))
       stop(mstyle$stop("Unknown 'na.action' specified under options()."))
@@ -24,8 +24,8 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    if (missing(atransf))
       atransf <- FALSE
 
-   transf.char  <- deparse(substitute(transf))
-   atransf.char <- deparse(substitute(atransf))
+   transf.char  <- deparse(transf)
+   atransf.char <- deparse(atransf)
 
    if (is.function(transf) && is.function(atransf))
       stop(mstyle$stop("Use either 'transf' or 'atransf' to specify a transformation (not both)."))
@@ -36,8 +36,13 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    if (missing(at))
       at <- NULL
 
-   if (missing(ilab))
+   mf <- match.call()
+
+   if (missing(ilab)) {
       ilab <- NULL
+   } else {
+      ilab <- .getx("ilab", mf=mf, data=x$data)
+   }
 
    if (missing(ilab.xpos))
       ilab.xpos <- NULL
@@ -45,14 +50,29 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    if (missing(ilab.pos))
       ilab.pos <- NULL
 
-   if (missing(order))
+   if (missing(order)) {
       order <- NULL
+   } else {
+      order <- .getx("order", mf=mf, data=x$data)
+   }
 
-   if (missing(colout))
+   if (missing(colout)) {
       colout <- "black"
+   } else {
+      colout <- .getx("colout", mf=mf, data=x$data)
+   }
 
-   if (missing(psize))
+   if (missing(pch)) {
+      pch <- 15
+   } else {
+      pch <- .getx("pch", mf=mf, data=x$data)
+   }
+
+   if (missing(psize)) {
       psize <- NULL
+   } else {
+      psize <- .getx("psize", mf=mf, data=x$data)
+   }
 
    if (missing(cex))
       cex <- NULL
@@ -63,7 +83,7 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    if (missing(cex.axis))
       cex.axis <- NULL
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
 
    ### digits[1] for annotations, digits[2] for x-axis labels
    ### note: digits can also be a list (e.g., digits=list(2L,3)); trailing 0's are dropped for intergers
@@ -71,6 +91,8 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    if (length(digits) == 1L)
       digits <- c(digits,digits)
 
+   ddd <- list(...)
+
    ############################################################################
 
    ### set default colors if user has not specified 'col' and 'border' arguments
@@ -111,19 +133,22 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    ### vertical expansion factor: 1st = CI end lines, 2nd = arrows, 3rd = summary polygon or fitted polygons
 
    if (length(efac) == 1L)
-      efac <- rep(efac, 3)
+      efac <- rep(efac, 3L)
 
    if (length(efac) == 2L)
       efac <- c(efac[1], efac[1], efac[2]) # if 2 values specified: 1st = CI end lines and arrows, 2nd = summary polygon or fitted polygons
 
    ### annotation symbols vector
 
-   if (missing(annosym))
+   if (is.null(ddd$annosym)) {
       annosym <- c(" [", ", ", "]", "-") # 4th element for minus sign symbol
-   if (length(annosym) == 3L)
-      annosym <- c(annosym, "-")
-   if (length(annosym) != 4L)
-      stop(mstyle$stop("Argument 'annosym' must be a vector of length 3."))
+   } else {
+      annosym <- ddd$annosym
+      if (length(annosym) == 3L)
+         annosym <- c(annosym, "-")
+      if (length(annosym) != 4L)
+         stop(mstyle$stop("Argument 'annosym' must be a vector of length 3 (or 4)."))
+   }
 
    ### get measure from object
 
@@ -133,7 +158,7 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
 
    estlab <- .setlab(measure, transf.char, atransf.char, gentype=3, short=TRUE)
    if (is.expression(estlab)) {
-      header.right <- parse(text=paste0("bold(", estlab, " * '", annosym[1], "' * '", 100*(1-level), "% CI'", " * '", annosym[3], "')"))
+      header.right <- str2lang(paste0("bold(", estlab, " * '", annosym[1], "' * '", 100*(1-level), "% CI'", " * '", annosym[3], "')"))
    } else {
       header.right <- paste0(estlab, annosym[1], 100*(1-level), "% CI", annosym[3])
    }
@@ -159,8 +184,6 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    if (!annotate)
       header.right <- NULL
 
-   ddd <- list(...)
-
    if (!is.null(ddd$addcred))
       addpred <- ddd$addcred
 
@@ -179,14 +202,46 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    if (!is.null(ddd$clim))
       olim <- ddd$clim
 
-   lplot     <- function(..., textpos, addcred, pi.type, decreasing, clim) plot(...)
-   labline   <- function(..., textpos, addcred, pi.type, decreasing, clim) abline(...)
-   lsegments <- function(..., textpos, addcred, pi.type, decreasing, clim) segments(...)
-   laxis     <- function(..., textpos, addcred, pi.type, decreasing, clim) axis(...)
-   lmtext    <- function(..., textpos, addcred, pi.type, decreasing, clim) mtext(...)
-   lpolygon  <- function(..., textpos, addcred, pi.type, decreasing, clim) polygon(...)
-   ltext     <- function(..., textpos, addcred, pi.type, decreasing, clim) text(...)
-   lpoints   <- function(..., textpos, addcred, pi.type, decreasing, clim) points(...)
+   ### row adjustments for 1) study labels, 2) annotations, and 3) ilab elements
+
+   if (is.null(ddd$rowadj)) {
+      rowadj <- rep(0,3)
+   } else {
+      rowadj <- ddd$rowadj
+      if (length(rowadj) == 1L)
+         rowadj <- c(rowadj,rowadj,0) # if one value is specified, use it for both 1&2
+      if (length(rowadj) == 2L)
+         rowadj <- c(rowadj,0) # if two values are specified, use them for 1&2
+   }
+
+   if (is.null(ddd$top)) {
+      top <- 3
+   } else {
+      top <- ddd$top
+   }
+
+   lplot     <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) plot(...)
+   labline   <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) abline(...)
+   lsegments <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) segments(...)
+   laxis     <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) axis(...)
+   lmtext    <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) mtext(...)
+   lpolygon  <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) polygon(...)
+   ltext     <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) text(...)
+   lpoints   <- function(..., textpos, addcred, pi.type, decreasing, clim, rowadj, annosym, top) points(...)
+
+   if (is.character(showweights)) {
+      weighttype  <- match.arg(showweights, c("diagonal", "rowsum"))
+      if (weighttype == "rowsum" && !inherits(x, "rma.mv"))
+         weighttype <- "diagonal"
+      if (weighttype == "rowsum" && !x$int.only)
+         stop(mstyle$stop("Row-sum weights are only meaningful for intercept-only models."))
+      showweights <- TRUE
+   } else {
+      weighttype <- "diagonal"
+   }
+
+   if (!is.logical(showweights))
+      stop(mstyle$stop("Argument 'showweights' must be a logical."))
 
    ### TODO: remove this when there is a weights() function for 'rma.glmm' objects
    if (inherits(x, "rma.glmm") && showweights)
@@ -224,7 +279,9 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
 
    } else {
 
-      if (is.null(slab) || (length(slab) == 1L && is.na(slab))) # slab=NULL or slab=NA can be used to suppress study labels
+      slab <- .getx("slab", mf=mf, data=x$data)
+
+      if (length(slab) == 1L && is.na(slab))    # slab=NA can be used to suppress study labels
          slab <- rep("", x$k.all)
 
       if (length(slab) != x$k.all)
@@ -299,7 +356,7 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
          }
       }
 
-      weights <- try(weights(x), silent=TRUE) # does not work for rma.glmm and rma.uni.selmodel objects
+      weights <- try(weights(x, type=weighttype), silent=TRUE) # does not work for rma.glmm and rma.uni.selmodel objects
 
       if (inherits(weights, "try-error"))
          weights <- rep(1, k)
@@ -332,7 +389,7 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
          if (length(order) != x$k.all)
             stop(mstyle$stop(paste0("Length of the 'order' argument (", length(order), ") does not correspond to the size of the original dataset (", x$k.all, ").")))
 
-         if (grepl("^order\\(", deparse(substitute(order)))) {
+         if (grepl("^order\\(", deparse1(substitute(order)))) {
             sort.vec <- order
          } else {
             sort.vec <- order(order, decreasing=decreasing)
@@ -570,17 +627,20 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
 
    ### allow adjustment of position of study labels and annotations via textpos argument
 
-   if (is.null(ddd$textpos))
-      ddd$textpos <- c(xlim[1], xlim[2])
+   if (is.null(ddd$textpos)) {
+      textpos <- xlim
+   } else {
+      textpos <- ddd$textpos
+   }
 
-   if (length(ddd$textpos) != 2L)
+   if (length(textpos) != 2L)
       stop(mstyle$stop("Argument 'textpos' must be of length 2."))
 
-   if (is.na(ddd$textpos[1]))
-      ddd$textpos[1] <- xlim[1]
+   if (is.na(textpos[1]))
+      textpos[1] <- xlim[1]
 
-   if (is.na(ddd$textpos[2]))
-      ddd$textpos[2] <- xlim[2]
+   if (is.na(textpos[2]))
+      textpos[2] <- xlim[2]
 
    ### set y-axis limits
 
@@ -628,16 +688,16 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
    ### when passing a named vector, the names are for 'family' and the values are for 'font'
 
    if (missing(fonts)) {
-      fonts <- rep(par("family"), 3)
+      fonts <- rep(par("family"), 3L)
    } else {
       if (length(fonts) == 1L)
-         fonts <- rep(fonts, 3)
+         fonts <- rep(fonts, 3L)
       if (length(fonts) == 2L)
          fonts <- c(fonts, fonts[1])
    }
 
    if (is.null(names(fonts)))
-      fonts <- structure(c(1L,1L,1L), names=fonts)
+      fonts <- setNames(c(1L,1L,1L), nm=fonts)
 
    par(family=names(fonts)[1], font=fonts[1])
 
@@ -813,9 +873,9 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
          mlab <- sapply(x$method, switch, "FE"="FE Model", "EE"="EE Model", "CE"="CE Model", "RE Model", USE.NAMES=FALSE)
 
       if (is.list(mlab)) {
-         ltext(ddd$textpos[1], -1, mlab[[1]], pos=4, cex=cex, ...)
+         ltext(textpos[1], -1+rowadj[1], mlab[[1]], pos=4, cex=cex, ...)
       } else {
-         ltext(ddd$textpos[1], -1, mlab, pos=4, cex=cex, ...)
+         ltext(textpos[1], -1+rowadj[1], mlab, pos=4, cex=cex, ...)
       }
 
    }
@@ -871,7 +931,7 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
 
    ### add study labels on the left
 
-   ltext(ddd$textpos[1], rows, slab, pos=4, cex=cex, ...)
+   ltext(textpos[1], rows+rowadj[1], slab, pos=4, cex=cex, ...)
 
    ### add info labels
 
@@ -884,7 +944,7 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
          ilab.pos <- rep(ilab.pos, ncol(ilab))
       par(family=names(fonts)[3], font=fonts[3])
       for (l in seq_len(ncol(ilab))) {
-         ltext(ilab.xpos[l], rows, ilab[,l], pos=ilab.pos[l], cex=cex, ...)
+         ltext(ilab.xpos[l], rows+rowadj[3], ilab[,l], pos=ilab.pos[l], cex=cex, ...)
       }
       par(family=names(fonts)[1], font=fonts[1])
    }
@@ -942,12 +1002,17 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
       } else {
          if (length(width) == 1L)
             width <- rep(width, ncol(annotext))
+         if (length(width) != ncol(annotext))
+            stop(mstyle$stop(paste0("Length of 'width' argument (", length(width), ") does not match the number of annotation columns (", ncol(annotext), ").")))
       }
 
       for (j in seq_len(ncol(annotext))) {
          annotext[,j] <- formatC(annotext[,j], width=width[j])
       }
 
+      if (showweights)
+         width <- width[-1] # remove the first entry for the weights (so this can be used by addpoly() via .metafor)
+
       if (showweights) {
          annotext <- cbind(annotext[,1], "%   ", annotext[,2], annosym[1], annotext[,3], annosym[2], annotext[,4], annosym[3])
       } else {
@@ -959,12 +1024,14 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
 
       par(family=names(fonts)[2], font=fonts[2])
       if (addfit && x$int.only) {
-         ltext(ddd$textpos[2], c(rows,-1), labels=annotext, pos=2, cex=cex, ...)
+         ltext(textpos[2], c(rows,-1)+rowadj[2], labels=annotext, pos=2, cex=cex, ...)
       } else {
-         ltext(ddd$textpos[2], rows, labels=annotext, pos=2, cex=cex, ...)
+         ltext(textpos[2], rows+rowadj[2], labels=annotext, pos=2, cex=cex, ...)
       }
       par(family=names(fonts)[1], font=fonts[1])
 
+   } else {
+      width <- NULL
    }
 
    ### add yi points
@@ -989,14 +1056,19 @@ lty, fonts, cex, cex.lab, cex.axis, annosym, ...) {
 
    ### add header
 
-   ltext(ddd$textpos[1], ylim[2]-(top-1)+1, header.left,  pos=4, font=2, cex=cex, ...)
-   ltext(ddd$textpos[2], ylim[2]-(top-1)+1, header.right, pos=2, font=2, cex=cex, ...)
+   ltext(textpos[1], ylim[2]-(top-1)+1, header.left,  pos=4, font=2, cex=cex, ...)
+   ltext(textpos[2], ylim[2]-(top-1)+1, header.right, pos=2, font=2, cex=cex, ...)
 
    #########################################################################
 
    ### return some information about plot invisibly
 
-   res <- list(xlim=par("usr")[1:2], alim=alim, at=at, ylim=ylim, rows=rows, cex=cex, cex.lab=cex.lab, cex.axis=cex.axis)
+   res <- list(xlim=par("usr")[1:2], alim=alim, at=at, ylim=ylim, rows=rows, cex=cex, cex.lab=cex.lab, cex.axis=cex.axis, ilab.xpos=ilab.xpos, ilab.pos=ilab.pos, textpos=textpos)
+
+   ### put stuff into the .metafor environment, so that it can be used by addpoly()
+
+   sav <- c(res, list(level=level, annotate=annotate, digits=digits[1], width=width, transf=transf, atransf=atransf, targs=targs, efac=efac[3], fonts=fonts[1:2], annosym=annosym))
+   try(assign("forest", sav, envir=.metafor), silent=TRUE)
 
    invisible(res)
 
