@@ -2,8 +2,7 @@ print.escalc <- function(x, digits=attr(x,"digits"), ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(x, "escalc"))
-      stop(mstyle$stop("Argument 'x' must be an object of class \"escalc\"."))
+   .chkclass(class(x), must="escalc")
 
    attr(x, "class") <- NULL
 
@@ -17,12 +16,19 @@ print.escalc <- function(x, digits=attr(x,"digits"), ...) {
    vi.pos    <- na.omit(match(attr(x, "vi.names"),    names(x)))
    sei.pos   <- na.omit(match(attr(x, "sei.names"),   names(x)))
    zi.pos    <- na.omit(match(attr(x, "zi.names"),    names(x)))
+   pval.pos  <- na.omit(match(attr(x, "pval.names"),  names(x)))
    ci.lb.pos <- na.omit(match(attr(x, "ci.lb.names"), names(x)))
    ci.ub.pos <- na.omit(match(attr(x, "ci.ub.names"), names(x)))
 
    ### get rownames attribute so we can back-assign it
 
    rnames <- attr(x, "row.names")
+
+   ### for printing, turn expressions into strings
+
+   is.expr <- sapply(x, is.expression)
+   x[is.expr] <- lapply(x[is.expr], as.character)
+
    x <- data.frame(x)
    rownames(x) <- rnames
 
@@ -40,13 +46,20 @@ print.escalc <- function(x, digits=attr(x,"digits"), ...) {
    if (length(zi.pos) > 0L)
       x[zi.pos] <- apply(x[zi.pos], 2, .fcf, digits[["test"]])
 
+   if (length(pval.pos) > 0L)
+      x[pval.pos] <- apply(x[pval.pos], 2, .pval, digits[["pval"]]) # note: using .pval here
+
    if (length(ci.lb.pos) > 0L)
       x[ci.lb.pos] <- apply(x[ci.lb.pos], 2, .fcf, digits[["ci"]])
 
    if (length(ci.ub.pos) > 0L)
       x[ci.ub.pos] <- apply(x[ci.ub.pos], 2, .fcf, digits[["ci"]])
 
+   .space()
+
    tmp <- capture.output(print(x, ...))
    .print.table(tmp, mstyle)
 
+   .space()
+
 }
