@@ -7,27 +7,26 @@ level=95, digits, verbose=FALSE, ...) {
 
    ###### setup
 
-   withcrayon <- "crayon" %in% .packages()
-   mstyle <- .get.mstyle(withcrayon)
+   mstyle <- .get.mstyle("crayon" %in% .packages())
 
    ### check argument specifications
 
-   if (length(add) == 1)
+   if (length(add) == 1L)
       add <- c(add, 0)
 
-   if (length(add) != 2)
+   if (length(add) != 2L)
       stop(mstyle$stop("Argument 'add' should specify one or two values (see 'help(rma.peto)')."))
 
-   if (length(to) == 1)
+   if (length(to) == 1L)
       to <- c(to, "none")
 
-   if (length(to) != 2)
+   if (length(to) != 2L)
       stop(mstyle$stop("Argument 'to' should specify one or two values (see 'help(rma.peto)')."))
 
-   if (length(drop00) == 1)
+   if (length(drop00) == 1L)
       drop00 <- c(drop00, FALSE)
 
-   if (length(drop00) != 2)
+   if (length(drop00) != 2L)
       stop(mstyle$stop("Argument 'drop00' should specify one or two values (see 'help(rma.peto)')."))
 
    na.act <- getOption("na.action")
@@ -41,17 +40,14 @@ level=95, digits, verbose=FALSE, ...) {
    if (!is.element(to[2], c("all","only0","if0all","none")))
       stop(mstyle$stop("Unknown 'to' argument specified."))
 
+   time.start <- proc.time()
+
    ### get ... argument and check for extra/superfluous arguments
 
    ddd <- list(...)
 
    .chkdots(ddd, c("outlist", "time"))
 
-   ### handle 'time' argument from ...
-
-   if (.isTRUE(ddd$time))
-      time.start <- proc.time()
-
    measure <- "PETO" ### set measure here so that it can be added below
 
    ### set defaults for digits
@@ -62,10 +58,19 @@ level=95, digits, verbose=FALSE, ...) {
       digits <- .set.digits(digits, dmiss=FALSE)
    }
 
+   ### set options(warn=1) if verbose > 2
+
+   if (verbose > 2) {
+      opwarn <- options(warn=1)
+      on.exit(options(warn=opwarn$warn), add=TRUE)
+   }
+
    #########################################################################
 
+   if (verbose) .space()
+
    if (verbose)
-      message(mstyle$message("\nExtracting data and computing yi/vi values ..."))
+      message(mstyle$message("Extracting data and computing yi/vi values ..."))
 
    ### check if data argument has been specified
 
@@ -83,25 +88,18 @@ level=95, digits, verbose=FALSE, ...) {
 
    ### extract slab and subset values, possibly from the data frame specified via data (arguments not specified are NULL)
 
-   mf.slab   <- mf[[match("slab",   names(mf))]]
-   mf.subset <- mf[[match("subset", names(mf))]]
-   slab   <- eval(mf.slab,   data, enclos=sys.frame(sys.parent()))
-   subset <- eval(mf.subset, data, enclos=sys.frame(sys.parent()))
+   slab   <- .getx("slab",   mf=mf, data=data)
+   subset <- .getx("subset", mf=mf, data=data)
 
    ### extract/calculate ai,bi,ci,di,n1i,n2i values
 
-   mf.ai  <- mf[[match("ai",  names(mf))]]
-   mf.bi  <- mf[[match("bi",  names(mf))]]
-   mf.ci  <- mf[[match("ci",  names(mf))]]
-   mf.di  <- mf[[match("di",  names(mf))]]
-   mf.n1i <- mf[[match("n1i", names(mf))]]
-   mf.n2i <- mf[[match("n2i", names(mf))]]
-   ai  <- eval(mf.ai,  data, enclos=sys.frame(sys.parent()))
-   bi  <- eval(mf.bi,  data, enclos=sys.frame(sys.parent()))
-   ci  <- eval(mf.ci,  data, enclos=sys.frame(sys.parent()))
-   di  <- eval(mf.di,  data, enclos=sys.frame(sys.parent()))
-   n1i <- eval(mf.n1i, data, enclos=sys.frame(sys.parent()))
-   n2i <- eval(mf.n2i, data, enclos=sys.frame(sys.parent()))
+   ai  <- .getx("ai",  mf=mf, data=data)
+   bi  <- .getx("bi",  mf=mf, data=data)
+   ci  <- .getx("ci",  mf=mf, data=data)
+   di  <- .getx("di",  mf=mf, data=data)
+   n1i <- .getx("n1i", mf=mf, data=data)
+   n2i <- .getx("n2i", mf=mf, data=data)
+
    if (is.null(bi)) bi <- n1i - ai
    if (is.null(di)) di <- n2i - ci
    ni <- ai + bi + ci + di
@@ -109,6 +107,9 @@ level=95, digits, verbose=FALSE, ...) {
    k <- length(ai) ### number of outcomes before subsetting
    k.all <- k
 
+   if (length(ai)==0L || length(bi)==0L || length(ci)==0L || length(di)==0L)
+      stop(mstyle$stop("Need to specify arguments ai, bi, ci, di (or ai, ci, n1i, n2i)."))
+
    ids <- seq_len(k)
 
    ### generate study labels if none are specified
@@ -129,6 +130,9 @@ level=95, digits, verbose=FALSE, ...) {
       if (length(slab) != k)
          stop(mstyle$stop("Study labels not of same length as data."))
 
+      if (is.factor(slab))
+         slab <- as.character(slab)
+
       slab.null <- FALSE
 
    }
@@ -140,6 +144,8 @@ level=95, digits, verbose=FALSE, ...) {
       if (verbose)
          message(mstyle$message("Subsetting ..."))
 
+      subset <- .setnafalse(subset, k=k)
+
       ai   <- ai[subset]
       bi   <- bi[subset]
       ci   <- ci[subset]
@@ -158,9 +164,9 @@ level=95, digits, verbose=FALSE, ...) {
 
    ### calculate observed effect estimates and sampling variances
 
-   dat <- escalc(measure="PETO", ai=ai, bi=bi, ci=ci, di=di, add=add[1], to=to[1], drop00=drop00[1])
-   yi  <- dat$yi ### one or more yi/vi pairs may be NA/NA
-   vi  <- dat$vi ### one or more yi/vi pairs may be NA/NA
+   dat <- .do.call(escalc, measure="PETO", ai=ai, bi=bi, ci=ci, di=di, add=add[1], to=to[1], drop00=drop00[1])
+   yi <- dat$yi ### one or more yi/vi pairs may be NA/NA
+   vi <- dat$vi ### one or more yi/vi pairs may be NA/NA
 
    ### if drop00[2]=TRUE, set counts to NA for studies that have no events (or all events) in both arms
 
@@ -201,7 +207,7 @@ level=95, digits, verbose=FALSE, ...) {
          ci   <- ci[not.na]
          di   <- di[not.na]
          k    <- length(ai)
-         warning(mstyle$warning("Tables with NAs omitted from model fitting."))
+         warning(mstyle$warning("Tables with NAs omitted from model fitting."), call.=FALSE)
       }
 
       if (na.act == "na.fail")
@@ -229,7 +235,7 @@ level=95, digits, verbose=FALSE, ...) {
          yi <- yi[not.na.yivi]
          vi <- vi[not.na.yivi]
          ni <- ni[not.na.yivi]
-         warning(mstyle$warning("Some yi/vi values are NA."))
+         warning(mstyle$warning("Some yi/vi values are NA."), call.=FALSE)
 
          attr(yi, "measure") <- measure ### add measure attribute back
          attr(yi, "ni")      <- ni      ### add ni attribute back
@@ -241,7 +247,7 @@ level=95, digits, verbose=FALSE, ...) {
 
    }
 
-   k.yi <- length(yi) ### number of yi/vi pairs that are not NA (needed for QE df and fitstats calculation)
+   k.yi <- length(yi) ### number of yi/vi pairs that are not NA (needed for QE df and fit.stats calculation)
 
    ### add/to procedures for the 2x2 tables for the actual meta-analysis
    ### note: technically, nothing needs to be added, but Stata/RevMan add 1/2 by default for only0 studies (but drop studies with no/all events)
@@ -293,7 +299,7 @@ level=95, digits, verbose=FALSE, ...) {
 
    #########################################################################
 
-   level <- ifelse(level == 0, 1, ifelse(level >= 1, (100-level)/100, ifelse(level > .5, 1-level, level)))
+   level <- .level(level)
 
    ###### model fitting, test statistics, and confidence intervals
 
@@ -333,12 +339,12 @@ level=95, digits, verbose=FALSE, ...) {
 
    if (k.pos > 1) {
       QEp <- pchisq(QE, df=k.yi-1, lower.tail=FALSE)
-      I2 <- max(0, 100 * (QE - (k.yi-1)) / QE)
-      H2 <- QE / (k.yi-1)
+      I2  <- max(0, 100 * (QE - (k.yi-1)) / QE)
+      H2  <- QE / (k.yi-1)
    } else {
       QEp <- 1
-      I2 <- 0
-      H2 <- 1
+      I2  <- 0
+      H2  <- 1
    }
 
    wi  <- 1/vi
@@ -353,7 +359,11 @@ level=95, digits, verbose=FALSE, ...) {
 
    ll.ML     <- -1/2 * (k.yi)   * log(2*base::pi)                   - 1/2 * sum(log(vi))                      - 1/2 * RSS
    ll.REML   <- -1/2 * (k.yi-1) * log(2*base::pi) + 1/2 * log(k.yi) - 1/2 * sum(log(vi)) - 1/2 * log(sum(wi)) - 1/2 * RSS
-   dev.ML    <- -2 * (ll.ML - sum(dnorm(yi, mean=yi, sd=sqrt(vi), log=TRUE)))
+   if (any(vi <= 0)) {
+      dev.ML <- -2 * ll.ML
+   } else {
+      dev.ML <- -2 * (ll.ML - sum(dnorm(yi, mean=yi, sd=sqrt(vi), log=TRUE)))
+   }
    AIC.ML    <- -2 * ll.ML   + 2
    BIC.ML    <- -2 * ll.ML   + log(k.yi)
    AICc.ML   <- -2 * ll.ML   + 2 * max(k.yi, 3) / (max(k.yi, 3) - 2)
@@ -381,44 +391,65 @@ level=95, digits, verbose=FALSE, ...) {
    X.f       <- cbind(rep(1,k.f))
    intercept <- TRUE
    int.only  <- TRUE
+   btt       <- 1
+   m         <- 1
+   coef.na   <- c(X=FALSE)
 
    method    <- "FE"
    weighted  <- TRUE
    test      <- "z"
-   dfs       <- NA
+   ddf       <- NA
 
-   if (is.null(ddd$outlist)) {
+   if (is.null(ddd$outlist) || ddd$outlist == "nodata") {
 
       res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb,
-                  tau2=tau2,
-                  k=k, k.f=k.f, k.yi=k.yi, k.pos=k.pos, k.eff=k.eff, k.all=k.all, p=p, parms=parms,
-                  QE=QE, QEp=QEp, I2=I2, H2=H2,
-                  int.only=int.only,
+                  tau2=tau2, tau2.f=tau2,
+                  I2=I2, H2=H2,
+                  QE=QE, QEp=QEp,
+                  k=k, k.f=k.f, k.yi=k.yi, k.pos=k.pos, k.eff=k.eff, k.all=k.all, p=p, p.eff=p.eff, parms=parms,
+                  int.only=int.only, intercept=intercept, coef.na=coef.na,
                   yi=yi, vi=vi, yi.f=yi.f, vi.f=vi.f, X.f=X.f, ai=ai, bi=bi, ci=ci, di=di, ai.f=ai.f, bi.f=bi.f, ci.f=ci.f, di.f=di.f, ni=ni, ni.f=ni.f,
                   ids=ids, not.na=not.na, subset=subset, not.na.yivi=not.na.yivi, slab=slab, slab.null=slab.null,
-                  measure=measure, method=method, weighted=weighted, test=test, dfs=dfs, intercept=intercept, digits=digits, level=level,
+                  measure=measure, method=method, weighted=weighted,
+                  test=test, ddf=ddf, dfs=ddf, btt=btt, m=m,
+                  digits=digits, level=level,
                   add=add, to=to, drop00=drop00,
-                  fit.stats=fit.stats, formula.yi=NULL, formula.mods=NULL, version=packageVersion("metafor"), call=mf)
+                  fit.stats=fit.stats,
+                  formula.yi=NULL, formula.mods=NULL, version=packageVersion("metafor"), call=mf)
+
+      if (is.null(ddd$outlist))
+         res <- append(res, list(data=data), which(names(res) == "fit.stats"))
 
    }
 
-   if (.isTRUE(ddd$time)) {
-      time.end <- proc.time()
-      res$time <- unname(time.end - time.start)[3]
+   if (!is.null(ddd$outlist)) {
+
+      if (ddd$outlist == "minimal") {
+         res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb,
+                     tau2=tau2,
+                     I2=I2, H2=H2,
+                     QE=QE, QEp=QEp,
+                     k=k, k.pos=k.pos, k.eff=k.eff, p=p, p.eff=p.eff, parms=parms,
+                     int.only=int.only,
+                     measure=measure, method=method,
+                     test=test, ddf=ddf, dfs=ddf, btt=btt, m=m,
+                     digits=digits,
+                     fit.stats=fit.stats)
+      } else {
+         res <- eval(str2lang(paste0("list(", ddd$outlist, ")")))
+      }
+
+   }
+
+   time.end <- proc.time()
+   res$time <- unname(time.end - time.start)[3]
+
+   if (.isTRUE(ddd$time))
       .print.time(res$time)
-   }
 
    if (verbose || .isTRUE(ddd$time))
       cat("\n")
 
-   if (!is.null(ddd$outlist)) {
-      if (ddd$outlist == "minimal") {
-         res <- list(b=beta, beta=beta, se=se, zval=zval, pval=pval, ci.lb=ci.lb, ci.ub=ci.ub, vb=vb, digits=digits, k=k, k.pos=k.pos, k.eff=k.eff, p=p, parms=parms, fit.stats=fit.stats, QE=QE, QEp=QEp)
-      } else {
-         res <- eval(parse(text=paste0("list(", ddd$outlist, ")")))
-      }
-   }
-
    class(res) <- c("rma.peto", "rma")
    return(res)
 
