@@ -1,18 +1,8 @@
-simulate.rma <- function (object, nsim = 1, seed = NULL, ...) {
+simulate.rma <- function(object, nsim = 1, seed = NULL, olim, ...) {
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(object, "rma"))
-      stop(mstyle$stop("Argument 'object' must be an object of class \"rma\"."))
-
-   if (inherits(object, "rma.glmm"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.glmm\"."))
-
-   if (inherits(object, "rma.mh"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.mh\"."))
-
-   if (inherits(object, "rma.peto"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.peto\"."))
+   .chkclass(class(object), must="rma", notav=c("rma.glmm", "rma.mh", "rma.peto", "rma.uni.selmodel"))
 
    na.act <- getOption("na.action")
 
@@ -28,31 +18,36 @@ simulate.rma <- function (object, nsim = 1, seed = NULL, ...) {
       R.seed <- get(".Random.seed", envir = .GlobalEnv)
       set.seed(seed)
       RNGstate <- structure(seed, kind = as.list(RNGkind()))
-      on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
+      on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv), add=TRUE)
    }
 
+   #########################################################################
+
    ### fitted values
 
    ftd <- c(object$X %*% object$beta)
 
-   ### for rma.uni objects, just need rnorm() (note: this also covers rma.ls objects)
-
-   if (inherits(object, "rma.uni")) {
+   ### simulate for rma.uni (and rma.ls) objects
 
+   if (inherits(object, "rma.uni"))
       val <- replicate(nsim, rnorm(object$k, mean=ftd, sd=sqrt(object$vi + object$tau2)))
 
+   ### simulate for rma.mv objects
+
+   if (inherits(object, "rma.mv"))
+      val <- t(.mvrnorm(nsim, mu=ftd, Sigma=object$M))
+
+   ### apply observation/outcome limits if specified
+
+   if (!missing(olim)) {
+      if (length(olim) != 2L)
+         stop(mstyle$stop("Argument 'olim' must be of length 2."))
+      olim <- sort(olim)
+      val[val < olim[1]] <- olim[1]
+      val[val > olim[2]] <- olim[2]
    }
 
-   ### for rma.mv objects, need mvrnorm() from MASS
-
-   if (inherits(object, "rma.mv")) {
-
-      if (!requireNamespace("MASS", quietly=TRUE))
-         stop(mstyle$stop("Please install the 'MASS' package to simulate from this model."))
-
-      val <- replicate(nsim, MASS::mvrnorm(1, mu=ftd, Sigma=object$M))
-
-   }
+   #########################################################################
 
    res <- matrix(NA_real_, nrow=object$k.f, ncol=nsim)
    res[object$not.na,] <- val
