@@ -5,9 +5,15 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    ### check argument specifications
 
+   if (missing(measure))
+      stop(mstyle$stop("Must specify an effect size or outcome measure via the 'measure' argument."))
+
+   if (!is.character(measure))
+      stop(mstyle$stop("The 'measure' argument must be a character string."))
+
    if (!is.element(measure, c("RR","OR","PETO","RD","AS","PHI","YUQ","YUY","RTET", ### 2x2 table measures
                               "PBIT","OR2D","OR2DN","OR2DL",                       ### - transformations to SMD
-                              "MPRD","MPRR","MPOR","MPORC","MPPETO",               ### - measures for matched pairs data
+                              "MPRD","MPRR","MPOR","MPORC","MPPETO","MPORM",       ### - measures for matched pairs data
                               "IRR","IRD","IRSD",                                  ### two-group person-time data measures
                               "MD","SMD","SMDH","ROM",                             ### two-group mean/SD measures
                               "CVR","VR",                                          ### coefficient of variation ratio, variability ratio
@@ -16,7 +22,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
                               "PCOR","ZPCOR","SPCOR",                              ### partial and semi-partial correlations
                               "PR","PLN","PLO","PAS","PFT",                        ### single proportions (and transformations thereof)
                               "IR","IRLN","IRS","IRFT",                            ### single-group person-time data (and transformations thereof)
-                              "MN","MNLN","CVLN","SDLN",                           ### mean, log(mean), log(CV), log(SD)
+                              "MN","MNLN","CVLN","SDLN","SMN",                     ### mean, log(mean), log(CV), log(SD), standardized mean
                               "MC","SMCC","SMCR","SMCRH","ROMC","CVRC","VRC",      ### raw/standardized mean change, log(ROM), CVR, and VR for dependent samples
                               "ARAW","AHW","ABT")))                                ### alpha (and transformations thereof)
       stop(mstyle$stop("Unknown 'measure' specified."))
@@ -48,50 +54,61 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    ### get slab and subset arguments (will be NULL when unspecified)
 
-   mf.slab   <- mf[[match("slab",   names(mf))]]
-   mf.subset <- mf[[match("subset", names(mf))]]
-   slab   <- eval(mf.slab,   data, enclos=sys.frame(sys.parent()))
-   subset <- eval(mf.subset, data, enclos=sys.frame(sys.parent()))
+   slab   <- .getx("slab",   mf=mf, data=data)
+   subset <- .getx("subset", mf=mf, data=data)
 
    #########################################################################
    #########################################################################
    #########################################################################
 
-   if (is.element(measure, c("RR","OR","RD","AS","PETO","PHI","YUQ","YUY","RTET","PBIT","OR2D","OR2DN","OR2DL","MPRD","MPRR","MPOR","MPORC","MPPETO"))) {
+   if (is.element(measure, c("RR","OR","RD","AS","PETO","PHI","YUQ","YUY","RTET","PBIT","OR2D","OR2DN","OR2DL","MPRD","MPRR","MPOR","MPORC","MPPETO","MPORM"))) {
 
-      mf.ai  <- mf[[match("ai",  names(mf))]]
-      mf.bi  <- mf[[match("bi",  names(mf))]]
-      mf.ci  <- mf[[match("ci",  names(mf))]]
-      mf.di  <- mf[[match("di",  names(mf))]]
-      mf.n1i <- mf[[match("n1i", names(mf))]]
-      mf.n2i <- mf[[match("n2i", names(mf))]]
-      ai  <- eval(mf.ai,  data, enclos=sys.frame(sys.parent()))
-      bi  <- eval(mf.bi,  data, enclos=sys.frame(sys.parent()))
-      ci  <- eval(mf.ci,  data, enclos=sys.frame(sys.parent()))
-      di  <- eval(mf.di,  data, enclos=sys.frame(sys.parent()))
-      n1i <- eval(mf.n1i, data, enclos=sys.frame(sys.parent()))
-      n2i <- eval(mf.n2i, data, enclos=sys.frame(sys.parent()))
-      if (is.null(bi)) bi <- n1i - ai
-      if (is.null(di)) di <- n2i - ci
+      ai  <- .getx("ai",  mf=mf, data=data)
+      bi  <- .getx("bi",  mf=mf, data=data)
+      ci  <- .getx("ci",  mf=mf, data=data)
+      di  <- .getx("di",  mf=mf, data=data)
+      n1i <- .getx("n1i", mf=mf, data=data)
+      n2i <- .getx("n2i", mf=mf, data=data)
+
+      if (!.equal.length(ai, bi, ci, di, n1i, n2i))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
+
+      n1i.inc <- n1i != ai + bi
+      n2i.inc <- n2i != ci + di
+
+      if (any(n1i.inc, na.rm=TRUE))
+         stop(mstyle$stop("One or more 'n1i' values are not equal to 'ai + bi'."))
+      if (any(n2i.inc, na.rm=TRUE))
+         stop(mstyle$stop("One or more 'n2i' values are not equal to 'ci + di'."))
+
+      bi <- replmiss(bi, n1i-ai)
+      di <- replmiss(di, n2i-ci)
+
+      if (!.all.specified(ai, bi, ci, di))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
 
       k <- length(ai) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          ai <- ai[subset]
          bi <- bi[subset]
          ci <- ci[subset]
          di <- di[subset]
       }
 
-      if (length(ai)==0L || length(bi)==0L || length(ci)==0L || length(di)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
+      n1i <- ai + bi
+      n2i <- ci + di
 
-      if (!all(length(ai) == c(length(ai),length(bi),length(ci),length(di))))
-         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
+      if (any(c(ai > n1i, ci > n2i), na.rm=TRUE))
+         stop(mstyle$stop("One or more event counts are larger than the corresponding group sizes."))
 
       if (any(c(ai, bi, ci, di) < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more counts are negative."))
 
+      if (any(c(n1i < 0, n2i < 0), na.rm=TRUE))
+         stop(mstyle$stop("One or more group sizes are negative."))
+
       ni.u <- ai + bi + ci + di ### unadjusted total sample sizes
 
       ### if drop00=TRUE, set counts to NA for studies that have no events (or all events) in both arms
@@ -154,35 +171,32 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    if (is.element(measure, c("IRR","IRD","IRSD"))) {
 
-      mf.x1i <- mf[[match("x1i", names(mf))]]
-      mf.x2i <- mf[[match("x2i", names(mf))]]
-      mf.t1i <- mf[[match("t1i", names(mf))]]
-      mf.t2i <- mf[[match("t2i", names(mf))]]
-      x1i <- eval(mf.x1i, data, enclos=sys.frame(sys.parent()))
-      x2i <- eval(mf.x2i, data, enclos=sys.frame(sys.parent()))
-      t1i <- eval(mf.t1i, data, enclos=sys.frame(sys.parent()))
-      t2i <- eval(mf.t2i, data, enclos=sys.frame(sys.parent()))
+      x1i <- .getx("x1i", mf=mf, data=data)
+      x2i <- .getx("x2i", mf=mf, data=data)
+      t1i <- .getx("t1i", mf=mf, data=data)
+      t2i <- .getx("t2i", mf=mf, data=data)
+
+      if (!.all.specified(x1i, x2i, t1i, t2i))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+      if (!.equal.length(x1i, x2i, t1i, t2i))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
       k <- length(x1i) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          x1i <- x1i[subset]
          x2i <- x2i[subset]
          t1i <- t1i[subset]
          t2i <- t2i[subset]
       }
 
-      if (length(x1i)==0L || length(x2i)==0L || length(t1i)==0L || length(t2i)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-      if (!all(length(x1i) == c(length(x1i),length(x2i),length(t1i),length(t2i))))
-         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
-
       if (any(c(x1i, x2i) < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more counts are negative."))
 
-      if (any(c(t1i, t2i) < 0, na.rm=TRUE))
-         stop(mstyle$stop("One or more person-times are negative."))
+      if (any(c(t1i, t2i) <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more person-times are <= 0."))
 
       ni.u <- t1i + t2i ### unadjusted total sample sizes
 
@@ -238,22 +252,23 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    if (is.element(measure, c("MD","SMD","SMDH","ROM","RPB","RBIS","D2OR","D2ORN","D2ORL"))) {
 
-      mf.m1i  <- mf[[match("m1i",  names(mf))]]
-      mf.m2i  <- mf[[match("m2i",  names(mf))]]
-      mf.sd1i <- mf[[match("sd1i", names(mf))]]
-      mf.sd2i <- mf[[match("sd2i", names(mf))]]
-      mf.n1i  <- mf[[match("n1i",  names(mf))]]
-      mf.n2i  <- mf[[match("n2i",  names(mf))]]
-      m1i  <- eval(mf.m1i,  data, enclos=sys.frame(sys.parent()))
-      m2i  <- eval(mf.m2i,  data, enclos=sys.frame(sys.parent()))
-      sd1i <- eval(mf.sd1i, data, enclos=sys.frame(sys.parent()))
-      sd2i <- eval(mf.sd2i, data, enclos=sys.frame(sys.parent()))
-      n1i  <- eval(mf.n1i,  data, enclos=sys.frame(sys.parent()))
-      n2i  <- eval(mf.n2i,  data, enclos=sys.frame(sys.parent()))
+      m1i  <- .getx("m1i",  mf=mf, data=data)
+      m2i  <- .getx("m2i",  mf=mf, data=data)
+      sd1i <- .getx("sd1i", mf=mf, data=data)
+      sd2i <- .getx("sd2i", mf=mf, data=data)
+      n1i  <- .getx("n1i",  mf=mf, data=data)
+      n2i  <- .getx("n2i",  mf=mf, data=data)
+
+      if (!.all.specified(m1i, m2i, sd1i, sd2i, n1i, n2i))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+      if (!.equal.length(m1i, m2i, sd1i, sd2i, n1i, n2i))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
       k <- length(n1i) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          m1i  <- m1i[subset]
          m2i  <- m2i[subset]
          sd1i <- sd1i[subset]
@@ -262,17 +277,11 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
          n2i  <- n2i[subset]
       }
 
-      if (length(m1i)==0L || length(m2i)==0L || length(sd1i)==0L || length(sd2i)==0L || length(n1i)==0L || length(n2i)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-      if (!all(length(m1i) == c(length(m1i),length(m2i),length(sd1i),length(sd2i),length(n1i),length(n2i))))
-         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
-
       if (any(c(sd1i, sd2i) < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more standard deviations are negative."))
 
-      if (any(c(n1i, n2i) < 0, na.rm=TRUE))
-         stop(mstyle$stop("One or more sample sizes are negative."))
+      if (any(c(n1i, n2i) <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more group sizes are <= 0."))
 
       ni.u <- n1i + n2i ### unadjusted total sample sizes
 
@@ -282,29 +291,28 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    if (is.element(measure, c("COR","UCOR","ZCOR"))) {
 
-      mf.ri <- mf[[match("ri", names(mf))]]
-      mf.ni <- mf[[match("ni", names(mf))]]
-      ri <- eval(mf.ri, data, enclos=sys.frame(sys.parent()))
-      ni <- eval(mf.ni, data, enclos=sys.frame(sys.parent()))
+      ri <- .getx("ri", mf=mf, data=data)
+      ni <- .getx("ni", mf=mf, data=data)
+
+      if (!.all.specified(ri, ni))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+      if (!.equal.length(ri, ni))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
       k <- length(ri) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          ri <- ri[subset]
          ni <- ni[subset]
       }
 
-      if (length(ri)==0L || length(ni)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-      if (length(ri) != length(ni))
-         stop(mstyle$stop("Supplied data vectors are not of the same length."))
-
       if (any(abs(ri) > 1, na.rm=TRUE))
          stop(mstyle$stop("One or more correlations are > 1 or < -1."))
 
-      if (any(ni < 0, na.rm=TRUE))
-         stop(mstyle$stop("One or more sample sizes are negative."))
+      if (any(ni <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more sample sizes are <= 0."))
 
       ni.u <- ni ### unadjusted total sample sizes
 
@@ -314,31 +322,43 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    if (is.element(measure, c("PR","PLN","PLO","PAS","PFT"))) {
 
-      mf.xi <- mf[[match("xi", names(mf))]]
-      mf.mi <- mf[[match("mi", names(mf))]]
-      mf.ni <- mf[[match("ni", names(mf))]]
-      xi <- eval(mf.xi, data, enclos=sys.frame(sys.parent()))
-      mi <- eval(mf.mi, data, enclos=sys.frame(sys.parent()))
-      ni <- eval(mf.ni, data, enclos=sys.frame(sys.parent()))
-      if (is.null(mi)) mi <- ni - xi
+      xi <- .getx("xi", mf=mf, data=data)
+      mi <- .getx("mi", mf=mf, data=data)
+      ni <- .getx("ni", mf=mf, data=data)
+
+      if (!.equal.length(xi, mi, ni))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
+
+      ni.inc <- ni != xi + mi
+
+      if (any(ni.inc, na.rm=TRUE))
+         stop(mstyle$stop("One or more 'ni' values are not equal to 'xi + mi'."))
+
+      mi <- replmiss(mi, ni-xi)
+
+      if (!.all.specified(xi, mi))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
 
       k <- length(xi) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          xi <- xi[subset]
          mi <- mi[subset]
       }
 
-      if (length(xi)==0L || length(mi)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
+      ni <- xi + mi
 
-      if (length(xi) != length(mi))
-         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
+      if (any(xi > ni, na.rm=TRUE))
+         stop(mstyle$stop("One or more event counts are larger than the corresponding group sizes."))
 
       if (any(c(xi, mi) < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more counts are negative."))
 
-      ni.u <- xi + mi ### unadjusted total sample sizes
+      if (any(ni <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more group sizes are <= 0."))
+
+      ni.u <- ni ### unadjusted total sample sizes
 
       if (to == "all") {
 
@@ -383,29 +403,28 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    if (is.element(measure, c("IR","IRLN","IRS","IRFT"))) {
 
-      mf.xi <- mf[[match("xi", names(mf))]]
-      mf.ti <- mf[[match("ti", names(mf))]]
-      xi <- eval(mf.xi, data, enclos=sys.frame(sys.parent()))
-      ti <- eval(mf.ti, data, enclos=sys.frame(sys.parent()))
+      xi <- .getx("xi", mf=mf, data=data)
+      ti <- .getx("ti", mf=mf, data=data)
+
+      if (!.all.specified(xi, ti))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+      if (!.equal.length(xi, ti))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
       k <- length(xi) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          xi <- xi[subset]
          ti <- ti[subset]
       }
 
-      if (length(xi)==0L || length(ti)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-      if (length(xi) != length(ti))
-         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
-
       if (any(xi < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more counts are negative."))
 
-      if (any(ti < 0, na.rm=TRUE))
-         stop(mstyle$stop("One or more person-times are negative."))
+      if (any(ti <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more person-times are <= 0."))
 
       ni.u <- ti ### unadjusted total sample sizes
 
@@ -447,34 +466,32 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    #########################################################################
 
-   if (is.element(measure, c("MN","MNLN"))) {
+   if (is.element(measure, c("MN","MNLN","SMN"))) {
 
-      mf.mi  <- mf[[match("mi",  names(mf))]]
-      mf.sdi <- mf[[match("sdi", names(mf))]]
-      mf.ni  <- mf[[match("ni",  names(mf))]]
-      mi  <- eval(mf.mi,  data, enclos=sys.frame(sys.parent()))
-      sdi <- eval(mf.sdi, data, enclos=sys.frame(sys.parent()))
-      ni  <- eval(mf.ni,  data, enclos=sys.frame(sys.parent()))
+      mi  <- .getx("mi",  mf=mf, data=data)
+      sdi <- .getx("sdi", mf=mf, data=data)
+      ni  <- .getx("ni",  mf=mf, data=data)
+
+      if (!.all.specified(mi, sdi, ni))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+      if (!.equal.length(mi, sdi, ni))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
       k <- length(ni) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          mi  <- mi[subset]
          sdi <- sdi[subset]
          ni  <- ni[subset]
       }
 
-      if (length(mi)==0L || length(sdi)==0L || length(ni)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-      if (!all(length(mi) == c(length(mi),length(sdi),length(ni))))
-         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
-
       if (any(sdi < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more standard deviations are negative."))
 
-      if (any(ni < 0, na.rm=TRUE))
-         stop(mstyle$stop("One or more sample sizes are negative."))
+      if (any(ni <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more sample sizes are <= 0."))
 
       if (is.element(measure, c("MNLN","CVLN")) && any(mi < 0, na.rm=TRUE))
          stop(mstyle$stop("One or more means are negative."))
@@ -487,22 +504,35 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    if (is.element(measure, c("MC","SMCC","SMCR","SMCRH","ROMC","CVRC"))) {
 
-      mf.m1i  <- mf[[match("m1i",  names(mf))]]
-      mf.m2i  <- mf[[match("m2i",  names(mf))]]
-      mf.sd1i <- mf[[match("sd1i", names(mf))]]
-      mf.sd2i <- mf[[match("sd2i", names(mf))]] ### for SMCR, do not need to supply this
-      mf.ni   <- mf[[match("ni",   names(mf))]]
-      mf.ri   <- mf[[match("ri",   names(mf))]]
-      m1i  <- eval(mf.m1i,  data, enclos=sys.frame(sys.parent()))
-      m2i  <- eval(mf.m2i,  data, enclos=sys.frame(sys.parent()))
-      sd1i <- eval(mf.sd1i, data, enclos=sys.frame(sys.parent()))
-      sd2i <- eval(mf.sd2i, data, enclos=sys.frame(sys.parent()))
-      ni   <- eval(mf.ni,   data, enclos=sys.frame(sys.parent()))
-      ri   <- eval(mf.ri,   data, enclos=sys.frame(sys.parent()))
+      m1i  <- .getx("m1i",  mf=mf, data=data)
+      m2i  <- .getx("m2i",  mf=mf, data=data)
+      sd1i <- .getx("sd1i", mf=mf, data=data)
+      sd2i <- .getx("sd2i", mf=mf, data=data)
+      ri   <- .getx("ri",   mf=mf, data=data) ### for SMCR, do not need to supply this
+      ni   <- .getx("ni",   mf=mf, data=data)
 
       k <- length(m1i) ### number of outcomes before subsetting
 
+      if (is.element(measure, c("MC","SMCC","SMCRH","ROMC","CVRC"))) {
+
+         if (!.all.specified(m1i, m2i, sd1i, sd2i, ni, ri))
+            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+         if (!.equal.length(m1i, m2i, sd1i, sd2i, ni, ri))
+            stop(mstyle$stop("Supplied data vectors are not all of the same length."))
+
+      } else {
+
+         if (!.all.specified(m1i, m2i, sd1i, ni, ri))
+            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+         if (!.equal.length(m1i, m2i, sd1i, ni, ri))
+            stop(mstyle$stop("Supplied data vectors are not all of the same length."))
+
+      }
+
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          m1i  <- m1i[subset]
          m2i  <- m2i[subset]
          sd1i <- sd1i[subset]
@@ -512,34 +542,18 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
       }
 
       if (is.element(measure, c("MC","SMCC","SMCRH","ROMC","CVRC"))) {
-
-         if (length(m1i)==0L || length(m2i)==0L || length(sd1i)==0L || length(sd2i)==0L || length(ni)==0L || length(ri)==0L)
-            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-         if (!all(length(m1i) == c(length(m1i),length(m2i),length(sd1i),length(sd2i),length(ni),length(ri))))
-            stop(mstyle$stop("Supplied data vectors are not all of the same length."))
-
          if (any(c(sd1i, sd2i) < 0, na.rm=TRUE))
             stop(mstyle$stop("One or more standard deviations are negative."))
-
       } else {
-
-         if (length(m1i)==0L || length(m2i)==0L || length(sd1i)==0L || length(ni)==0L || length(ri)==0L)
-            stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-         if (!all(length(m1i) == c(length(m1i),length(m2i),length(sd1i),length(ni),length(ri))))
-            stop(mstyle$stop("Supplied data vectors are not all of the same length."))
-
          if (any(sd1i < 0, na.rm=TRUE))
             stop(mstyle$stop("One or more standard deviations are negative."))
-
       }
 
       if (any(abs(ri) > 1, na.rm=TRUE))
          stop(mstyle$stop("One or more correlations are > 1 or < -1."))
 
-      if (any(ni < 0, na.rm=TRUE))
-         stop(mstyle$stop("One or more sample sizes are negative."))
+      if (any(ni <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more sample sizes are <= 0."))
 
       ni.u <- ni ### unadjusted total sample sizes
 
@@ -549,35 +563,33 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    if (is.element(measure, c("ARAW","AHW","ABT"))) {
 
-      mf.ai <- mf[[match("ai", names(mf))]]
-      mf.mi <- mf[[match("mi", names(mf))]]
-      mf.ni <- mf[[match("ni", names(mf))]]
-      ai <- eval(mf.ai, data, enclos=sys.frame(sys.parent()))
-      mi <- eval(mf.mi, data, enclos=sys.frame(sys.parent()))
-      ni <- eval(mf.ni, data, enclos=sys.frame(sys.parent()))
+      ai <- .getx("ai", mf=mf, data=data)
+      mi <- .getx("mi", mf=mf, data=data)
+      ni <- .getx("ni", mf=mf, data=data)
+
+      if (!.all.specified(ai, mi, ni))
+         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required\n  information is specified via the appropriate arguments."))
+
+      if (!.equal.length(ai, mi, ni))
+         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
 
       k <- length(ai) ### number of outcomes before subsetting
 
       if (!is.null(subset)) {
+         subset <- .setnafalse(subset, k=k)
          ai <- ai[subset]
          mi <- mi[subset]
          ni <- ni[subset]
       }
 
-      if (length(ai)==0L || length(mi)==0L || length(ni)==0L)
-         stop(mstyle$stop("Cannot compute outcomes. Check that all of the required \n  information is specified via the appropriate arguments."))
-
-      if (!all(length(ai) == c(length(ai),length(mi),length(ni))))
-         stop(mstyle$stop("Supplied data vectors are not all of the same length."))
-
       if (any(ai > 1, na.rm=TRUE))
          stop(mstyle$stop("One or more alpha values are > 1."))
 
       if (any(mi < 2, na.rm=TRUE))
          stop(mstyle$stop("One or more mi values are < 2."))
 
-      if (any(ni < 0, na.rm=TRUE))
-         stop(mstyle$stop("One or more sample sizes are negative."))
+      if (any(ni <= 0, na.rm=TRUE))
+         stop(mstyle$stop("One or more sample sizes are <= 0."))
 
       ni.u <- ni ### unadjusted total sample sizes
 
@@ -620,7 +632,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
    #########################################################################
    #########################################################################
 
-   if (is.element(measure, c("RR","OR","RD","AS","PETO","PHI","YUQ","YUY","RTET","PBIT","OR2D","OR2DN","OR2DL"))) {
+   if (is.element(measure, c("RR","OR","RD","AS","PETO","PHI","YUQ","YUY","RTET","PBIT","OR2D","OR2DN","OR2DL","MPORM"))) {
 
       ### check for NAs in table data and act accordingly
 
@@ -636,7 +648,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             ci   <- ci[not.na]
             di   <- di[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -694,7 +706,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             ci   <- ci[not.na]
             di   <- di[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -752,7 +764,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             ci   <- ci[not.na]
             di   <- di[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -810,7 +822,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             t1i  <- t1i[not.na]
             t2i  <- t2i[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -870,7 +882,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             n1i  <- n1i[not.na]
             n2i  <- n2i[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -926,7 +938,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             ri   <- ri[not.na]
             ni   <- ni[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -982,7 +994,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             xi   <- xi[not.na]
             mi   <- mi[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1038,7 +1050,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             xi   <- xi[not.na]
             ti   <- ti[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1080,7 +1092,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
 
    #########################################################################
 
-   if (is.element(measure, c("MN","MNLN"))) {
+   if (is.element(measure, c("MN","MNLN","SMN"))) {
 
       ### check for NAs in table data and act accordingly
 
@@ -1095,7 +1107,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             sdi  <- sdi[not.na]
             ni   <- ni[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1160,7 +1172,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             ni   <- ni[not.na]
             ri   <- ri[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
@@ -1239,7 +1251,7 @@ data, slab, subset, add=1/2, to="none", drop00=FALSE, rows, cols) {
             mi   <- mi[not.na]
             ni   <- ni[not.na]
             slab <- slab[not.na]
-            warning(mstyle$warning("Tables with NAs omitted."))
+            warning(mstyle$warning("Tables with NAs omitted."), call.=FALSE)
          }
 
          if (na.act == "na.fail")
