@@ -4,14 +4,7 @@ trimfill.rma.uni <- function(x, side, estimator="L0", maxiter=100, verbose=FALSE
 
    mstyle <- .get.mstyle("crayon" %in% .packages())
 
-   if (!inherits(x, "rma.uni"))
-      stop(mstyle$stop("Argument 'x' must be an object of class \"rma.uni\"."))
-
-   if (inherits(x, "robust.rma"))
-      stop(mstyle$stop("Method not available for objects of class \"robust.rma\"."))
-
-   if (inherits(x, "rma.ls"))
-      stop(mstyle$stop("Method not available for objects of class \"rma.ls\"."))
+   .chkclass(class(x), must="rma.uni", notav=c("robust.rma", "rma.ls", "rma.uni.selmodel"))
 
    if (!x$int.only)
       stop(mstyle$stop("Trim-and-fill method only applicable for models without moderators."))
@@ -34,7 +27,8 @@ trimfill.rma.uni <- function(x, side, estimator="L0", maxiter=100, verbose=FALSE
    ### determine side (if none is specified)
 
    if (is.null(side)) {
-      res <- suppressWarnings(rma.uni(yi, vi, weights=wi, mods=sqrt(vi), method=x$method, weighted=x$weighted, ...))
+      args <- list(yi=yi, vi=vi, weights=wi, mods=sqrt(vi), method=x$method, weighted=x$weighted, outlist="beta=beta", ...)
+      res <- suppressWarnings(.do.call(rma.uni, args))
       ### TODO: add check in case there are problems with fitting the model
       if (res$beta[2] < 0) {
          side <- "right"
@@ -84,7 +78,8 @@ trimfill.rma.uni <- function(x, side, estimator="L0", maxiter=100, verbose=FALSE
       vi.t <- vi[seq_len(k-k0)]
       wi.t <- wi[seq_len(k-k0)]
 
-      res <- suppressWarnings(rma.uni(yi.t, vi.t, weights=wi.t, method=x$method, weighted=x$weighted, ...))
+      args <- list(yi=yi.t, vi=vi.t, weights=wi.t, method=x$method, weighted=x$weighted, outlist="beta=beta", ...)
+      res <- suppressWarnings(.do.call(rma.uni, args))
 
       ### intercept estimate based on truncated data
 
@@ -169,7 +164,8 @@ trimfill.rma.uni <- function(x, side, estimator="L0", maxiter=100, verbose=FALSE
 
       ### fit model with imputed data
 
-      res <- suppressWarnings(rma.uni(yi.fill, vi.fill, weights=wi.fill, ni=ni.fill, method=x$method, weighted=x$weighted, ...))
+      args <- list(yi=yi.fill, vi=vi.fill, weights=wi.fill, ni=ni.fill, method=x$method, weighted=x$weighted, digits=x$digits, ...)
+      res <- suppressWarnings(.do.call(rma.uni, args))
 
       ### fill, ids, and slab are of length 'k.f + k0' (i.e., subsetted but with NAs)
 
