@@ -4,7 +4,7 @@
 
 context("Checking analysis example: rothman2008")
 
-source("tolerances.r") # read in tolerances
+source("settings.r")
 
 ############################################################################
 
@@ -133,20 +133,20 @@ test_that("results are correct for Mantel-Haenszel method.", {
    skip_on_cran()
 
    ### conditional MLE of the odds ratio
-   res <- rma.glmm(ai=ai, bi=bi, ci=ci, di=di, data=dat, measure="OR", model="CM.EL", method="FE")
+   res <- rma.glmm(ai=ai, bi=bi, ci=ci, di=di, data=dat, measure="OR", model="CM.EL", method="EE")
 
    expect_equivalent(coef(res),  0.3381, tolerance=.tol[["coef"]])
-   expect_equivalent(res$ci.lb, -0.2707, tolerance=.tol[["ci"]])
-   expect_equivalent(res$ci.ub,  0.9468, tolerance=.tol[["ci"]])
-   expect_equivalent(res$QE.Wld,  0.3484, tolerance=.tol[["test"]])
-   expect_equivalent(res$QEp.Wld, 0.5550, tolerance=.tol[["pval"]])
+   expect_equivalent(res$ci.lb, -0.2699, tolerance=.tol[["ci"]])
+   expect_equivalent(res$ci.ub,  0.9461, tolerance=.tol[["ci"]])
+   expect_equivalent(res$QE.Wld,  0.3480, tolerance=.tol[["test"]])
+   expect_equivalent(res$QEp.Wld, 0.5552, tolerance=.tol[["pval"]])
    expect_equivalent(res$QE.LRT,  0.3502, tolerance=.tol[["test"]])
    expect_equivalent(res$QEp.LRT, 0.5540, tolerance=.tol[["pval"]])
 
    tmp <- predict(res, transf=exp)
    expect_equivalent(tmp$pred,  1.4022, tolerance=.tol[["pred"]])
-   expect_equivalent(tmp$ci.lb, 0.7629, tolerance=.tol[["ci"]])
-   expect_equivalent(tmp$ci.ub, 2.5774, tolerance=.tol[["ci"]])
+   expect_equivalent(tmp$ci.lb, 0.7634, tolerance=.tol[["ci"]])
+   expect_equivalent(tmp$ci.ub, 2.5756, tolerance=.tol[["ci"]])
 
 })
 
@@ -269,7 +269,7 @@ test_that("results are correct for Mantel-Haenszel method.", {
    skip_on_cran()
 
    ### unconditional MLE of the rate ratio
-   res <- rma.glmm(x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i, data=dat, measure="IRR", digits=2, level=90, model="UM.FS", method="FE")
+   res <- rma.glmm(x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i, data=dat, measure="IRR", digits=2, level=90, model="UM.FS", method="EE")
 
    expect_equivalent(coef(res), 0.3545, tolerance=.tol[["coef"]])
    expect_equivalent(res$ci.lb, 0.1779, tolerance=.tol[["ci"]])
@@ -285,7 +285,7 @@ test_that("results are correct for Mantel-Haenszel method.", {
    expect_equivalent(tmp$ci.ub, 1.7009, tolerance=.tol[["ci"]])
 
    ### conditional MLE of the rate ratio
-   res <- rma.glmm(x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i, data=dat, measure="IRR", digits=2, level=90, model="CM.EL", method="FE")
+   res <- rma.glmm(x1i=x1i, x2i=x2i, t1i=t1i, t2i=t2i, data=dat, measure="IRR", digits=2, level=90, model="CM.EL", method="EE")
 
    expect_equivalent(coef(res), 0.3545, tolerance=.tol[["coef"]])
    expect_equivalent(res$ci.lb, 0.1779, tolerance=.tol[["ci"]])
@@ -363,7 +363,7 @@ test_that("results are correct for Mantel-Haenszel method.", {
    skip_on_cran()
 
    ### unconditional MLE of the odds ratio
-   res <- rma.glmm(ai=ai, bi=bi, ci=ci, di=di, data=dat, measure="OR", digits=2, level=90, model="UM.FS", method="FE")
+   res <- rma.glmm(ai=ai, bi=bi, ci=ci, di=di, data=dat, measure="OR", digits=2, level=90, model="UM.FS", method="EE")
 
    expect_equivalent(coef(res), 1.3318, tolerance=.tol[["coef"]])
    expect_equivalent(res$ci.lb, 0.3582, tolerance=.tol[["ci"]])
@@ -379,21 +379,23 @@ test_that("results are correct for Mantel-Haenszel method.", {
    expect_equivalent(tmp$ci.ub, 10.0276, tolerance=.tol[["ci"]])
 
    ### conditional MLE of the odds ratio
-   res <- rma.glmm(ai=ai, bi=bi, ci=ci, di=di, data=dat, measure="OR", digits=2, level=90, model="CM.EL", method="FE")
+   res <- rma.glmm(ai=ai, bi=bi, ci=ci, di=di, data=dat, measure="OR", digits=2, level=90, model="CM.EL", method="EE", control=list(optimizer="bobyqa"))
 
    expect_equivalent(coef(res), 1.3257, tolerance=.tol[["coef"]])
-   expect_equivalent(res$ci.lb, 0.3551, tolerance=.tol[["ci"]])
-   expect_equivalent(res$ci.ub, 2.2962, tolerance=.tol[["ci"]])
-   expect_equivalent(res$QE.Wld,  0.1237, tolerance=.tol[["test"]])
-   expect_equivalent(res$QEp.Wld, 0.7250, tolerance=.tol[["pval"]])
+   expect_equivalent(res$ci.lb, 0.3563, tolerance=.tol[["ci"]])
+   expect_equivalent(res$ci.ub, 2.2950, tolerance=.tol[["ci"]])
+   expect_equivalent(res$QE.Wld,  0.1556, tolerance=.tol[["test"]])
+   expect_equivalent(res$QEp.Wld, 0.6932, tolerance=.tol[["pval"]])
    expect_equivalent(res$QE.LRT,  0.1188, tolerance=.tol[["test"]])
    expect_equivalent(res$QEp.LRT, 0.7304, tolerance=.tol[["pval"]])
 
    tmp <- predict(res, transf=exp)
    expect_equivalent(tmp$pred,  3.7647, tolerance=.tol[["pred"]])
-   expect_equivalent(tmp$ci.lb, 1.4264, tolerance=.tol[["ci"]])
-   expect_equivalent(tmp$ci.ub, 9.9361, tolerance=.tol[["ci"]])
+   expect_equivalent(tmp$ci.lb, 1.4280, tolerance=.tol[["ci"]])
+   expect_equivalent(tmp$ci.ub, 9.9246, tolerance=.tol[["ci"]])
 
 })
 
 ############################################################################
+
+rm(list=ls())
