@@ -2,9 +2,12 @@
 
 context("Checking misc: plot() function")
 
+source("settings.r")
+
 test_that("plot can be drawn for rma().", {
 
-   data(dat.bcg, package="metafor")
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
+
    dat <- escalc(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
    res <- rma(yi, vi, data=dat)
    plot(res)
@@ -16,7 +19,8 @@ test_that("plot can be drawn for rma().", {
 
 test_that("plot can be drawn for rma.mh().", {
 
-   data(dat.bcg, package="metafor")
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
+
    res <- rma.mh(measure="RR", ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
    plot(res)
 
@@ -24,8 +28,11 @@ test_that("plot can be drawn for rma.mh().", {
 
 test_that("plot can be drawn for rma.peto().", {
 
-   data(dat.bcg, package="metafor")
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
+
    res <- rma.peto(ai=tpos, bi=tneg, ci=cpos, di=cneg, data=dat.bcg)
    plot(res)
 
 })
+
+rm(list=ls())
