@@ -1,18 +1,19 @@
 ### library(metafor); library(testthat); Sys.setenv(NOT_CRAN="true")
 
-### see also: http://www.metafor-project.org/doku.php/plots:plot_of_influence_diagnostics
+### see also: https://www.metafor-project.org/doku.php/plots:plot_of_influence_diagnostics
+
+source("settings.r")
 
 context("Checking plots example: plot of influence diagnostics")
 
 test_that("plot can be drawn.", {
 
+   expect_equivalent(TRUE, TRUE) # avoid 'Empty test' message
+
    skip_on_cran()
 
    opar <- par(no.readonly=TRUE)
 
-   ### load validity of employment interviews data
-   data(dat.mcdaniel1994, package="metafor")
-
    ### fit random-effects model with r-to-z transformed correlations
    res <- rma(ri=ri, ni=ni, measure="ZCOR", data=dat.mcdaniel1994)
 
@@ -28,3 +29,5 @@ test_that("plot can be drawn.", {
    par(opar)
 
 })
+
+rm(list=ls())
